@@ -1,5 +1,7 @@
 using System;
 using System.IO;
+using FSpot.Utils;
+using Mono.Unix;
 
 namespace FSpot {
 	public class ImageFormatException : ApplicationException {
@@ -15,7 +17,7 @@ namespace FSpot {
 
 		public ImageFile (string path) 
 		{
-			this.uri = UriList.PathToFileUri (path);
+			this.uri = UriUtils.PathToFileUri (path);
 		}
 		
 		public ImageFile (Uri uri)
@@ -31,7 +33,7 @@ namespace FSpot {
 			//if (vfs.IsLocal)
 			//	return File.OpenRead (uri.LocalPath);
 
-			System.Console.WriteLine ("open uri = {0}", uri.ToString ());
+			Log.DebugFormat ("open uri = {0}", uri.ToString ());
 			return new Gnome.Vfs.VfsStream (uri.ToString (), FileMode.Open);
 		}
 
@@ -82,7 +84,7 @@ namespace FSpot {
 		
 		public virtual void Save (Gdk.Pixbuf pixbuf, System.IO.Stream stream)
 		{
-			throw new NotImplementedException ();
+			throw new NotImplementedException (Catalog.GetString ("Writing to this file format is not supported"));
 		}
 
 		protected Gdk.Pixbuf TransformAndDispose (Gdk.Pixbuf orig)
@@ -111,7 +113,7 @@ namespace FSpot {
 			System.IO.Stream stream = PixbufStream ();
 			if (stream == null) {
 				Gdk.Pixbuf orig = this.Load ();
-				Gdk.Pixbuf scaled = PixbufUtils.ScaleToMaxSize (orig,  max_width, max_height);	
+				Gdk.Pixbuf scaled = PixbufUtils.ScaleToMaxSize (orig,  max_width, max_height, false);
 				orig.Dispose ();
 				return scaled;
 			}
@@ -150,9 +152,10 @@ namespace FSpot {
 			}
 		}
 
+		[Obsolete ("use HasLoader (System.Uri) instead")]
 		public static bool HasLoader (string path)
 		{
-			return HasLoader (UriList.PathToFileUri (path));
+			return HasLoader (UriUtils.PathToFileUri (path));
 		}
 		
 		public static bool HasLoader (Uri uri)
@@ -164,9 +167,10 @@ namespace FSpot {
 			return (t != null);
 		}
 
+		[Obsolete ("use Create (System.Uri) instead")]
 		public static ImageFile Create (string path)
 		{
-			return Create (UriList.PathToFileUri (path));
+			return Create (UriUtils.PathToFileUri (path));
 		}
 
 		public static ImageFile Create (Uri uri)
@@ -215,7 +219,17 @@ namespace FSpot {
 
 		public static bool IsRaw (string name)
 		{
-			string [] raw_extensions = {".nef", ".crw", ".cr2", ".arw", ".mrw", ".pef", ".dng"};
+			string [] raw_extensions = {
+				".arw",
+				".crw",
+				".cr2",
+				".dng",
+				".mrw",
+				".nef",
+				".orf", 
+				".pef", 
+				".raw",
+			};
 			foreach (string ext in raw_extensions)
 				if (ext == System.IO.Path.GetExtension (name).ToLower ())
 					return true;
