@@ -8,7 +8,6 @@
  */
 
 //TODO:
-//	* deal with vertical orientations (medium)
 //	* only redraw required parts on ExposeEvents (low)
 //	* Handle orientation changes (low) (require gtk# changes, so I can trigger an OrientationChanged event)
 
@@ -18,12 +17,14 @@ using System.Collections;
 using Gtk;
 using Gdk;
 
+using FSpot.Utils;
+
 namespace FSpot.Widgets
 {
 	public class Filmstrip : EventBox, IDisposable
 	{
 
-		public event OrientationChangedHandler OrientationChanged;
+//		public event OrientationChangedHandler OrientationChanged;
 		public event EventHandler PositionChanged;
 
 		bool extendable = true;
@@ -40,11 +41,11 @@ namespace FSpot.Widgets
 					return;
 
 				throw new NotImplementedException ();
-				if (OrientationChanged != null) {
-					OrientationChangedArgs args = new OrientationChangedArgs ();
-					//args.Orientation = value;
-					//OrientationChanged (this, args);
-				}
+//				if (OrientationChanged != null) {
+//					OrientationChangedArgs args = new OrientationChangedArgs ();
+//					//args.Orientation = value;
+//					//OrientationChanged (this, args);
+//				}
 			}
 		}
 
@@ -207,7 +208,7 @@ namespace FSpot.Widgets
 			}
 		}
 
-		int x_offset = 2, y_offset = 2;
+		int x_offset = 2;
 		public int XOffset {
 			get { return x_offset; }
 			set { 
@@ -217,6 +218,7 @@ namespace FSpot.Widgets
 			}
 		}
 
+		int y_offset = 2;
 		public int YOffset {
 			get { return y_offset; }
 			set { 
@@ -274,7 +276,16 @@ namespace FSpot.Widgets
 
 		public int ActiveItem {
 			get { return selection.Index; }
-			set { selection.Index = value; }
+			set {
+				if (value == selection.Index)
+					return;
+				if (value < 0)
+					value = 0;
+				if (value > selection.Collection.Count - 1)
+					value = selection.Collection.Count - 1;
+
+				selection.Index = value;
+			}
 		}
 
 		float position;
@@ -298,9 +309,11 @@ namespace FSpot.Widgets
 		}
 
 		FSpot.BrowsablePointer selection;
-		ThumbnailCache thumb_cache;
+		DisposableCache<string, Pixbuf> thumb_cache;
 
-		public Filmstrip (FSpot.BrowsablePointer selection) : this (selection, true) { }
+		public Filmstrip (FSpot.BrowsablePointer selection) : this (selection, true)
+		{
+		}
 
 		public Filmstrip (FSpot.BrowsablePointer selection, bool squared_thumbs) : base ()
 		{
@@ -310,8 +323,8 @@ namespace FSpot.Widgets
 			this.selection.Collection.Changed += HandleCollectionChanged;
 			this.selection.Collection.ItemsChanged += HandleCollectionItemsChanged;
 			this.squared_thumbs = squared_thumbs;
-			thumb_cache = new ThumbnailCache (30);
-			ThumbnailGenerator.Default.OnPixbufLoaded += delegate (PixbufLoader pl, string path, int order, Pixbuf p) {QueueDraw ();};
+			thumb_cache = new DisposableCache<string, Pixbuf> (30);
+			ThumbnailGenerator.Default.OnPixbufLoaded += HandlePixbufLoaded;
 		}
 	
 		int min_length = 400;
@@ -322,7 +335,6 @@ namespace FSpot.Widgets
 				requisition.Width += BackgroundTile.Width - min_length % BackgroundTile.Width;	
 
 			requisition.Height = BackgroundTile.Height + (2 * y_offset);
-			base.OnSizeRequested(ref requisition);
 		}
 
 		Pixbuf background_pixbuf;
@@ -353,7 +365,6 @@ namespace FSpot.Widgets
 			}
 		}
 
-	
 		Hashtable start_indexes;
 		int filmstrip_end_pos;
 		protected override bool OnExposeEvent (EventExpose evnt)
@@ -436,6 +447,24 @@ namespace FSpot.Widgets
 			return false;
 		}
 
+		protected override bool OnKeyPressEvent (Gdk.EventKey ek)
+		{
+			switch (ek.Key) {
+			case Gdk.Key.Page_Down:
+			case Gdk.Key.Down:
+			case Gdk.Key.Right:
+				ActiveItem ++;
+				return true;
+				
+			case Gdk.Key.Page_Up:
+			case Gdk.Key.Up:
+			case Gdk.Key.Left:
+				ActiveItem --;
+				return true;
+			}
+			return false;
+		}
+
 		public delegate void PositionChangedHandler (float position);
 
 		protected virtual void OnPositionChanged (float position)
@@ -444,27 +473,40 @@ namespace FSpot.Widgets
 			QueueDraw ();
 		}
 
-		public void HandlePointerChanged (BrowsablePointer pointer, BrowsablePointerChangedArgs old)
+		void HandlePointerChanged (BrowsablePointer pointer, BrowsablePointerChangedArgs old)
 		{
 			Position = ActiveItem;
 		}
 
-		public void HandleCollectionChanged (IBrowsableCollection coll)
+		void HandleCollectionChanged (IBrowsableCollection coll)
 		{
 			Position = ActiveItem;
 			QueueDraw ();
 		}
 
-		public void HandleCollectionItemsChanged (IBrowsableCollection coll, BrowsableEventArgs args)
+		void HandleCollectionItemsChanged (IBrowsableCollection coll, BrowsableEventArgs args)
 		{
-			//FIXME: need to be smarter here...
+			if (!args.Changes.DataChanged)
+				return;
+			foreach (int item in args.Items)
+				thumb_cache.TryRemove (FSpot.ThumbnailGenerator.ThumbnailPath ((selection.Collection [item]).DefaultVersionUri));
 
-			//invalidate the thumbs cache
-			thumb_cache.Dispose ();
-			thumb_cache = new ThumbnailCache (30);
+			//FIXME call QueueDrawArea
 			QueueDraw ();
 		}
 
+		void HandlePixbufLoaded (PixbufLoader pl, Uri uri, int order, Pixbuf p) {
+			if (!thumb_cache.Contains (FSpot.ThumbnailGenerator.ThumbnailPath (uri))) {
+				return;
+			}
+			
+			//FIXME use QueueDrawArea
+			//FIXME only invalidate if displayed
+			QueueDraw ();
+			
+
+		}
+
 		protected override bool OnButtonPressEvent (EventButton evnt)
 		{
 			if(evnt.Button != 1 || evnt.X > filmstrip_end_pos) 
@@ -493,7 +535,7 @@ namespace FSpot.Widgets
 			Pixbuf current;
 			try {
 				thumb_path = FSpot.ThumbnailGenerator.ThumbnailPath ((selection.Collection [i]).DefaultVersionUri);
-				current = thumb_cache.GetThumbnailForPath (thumb_path);
+				current = PixbufUtils.ShallowCopy (thumb_cache.Get (thumb_path));
 			} catch (IndexOutOfRangeException) {
 				thumb_path = null;
 				current = null;
@@ -508,16 +550,24 @@ namespace FSpot.Widgets
 						current = PixbufUtils.IconFromPixbuf (current, ThumbSize);
 					} else 
 						current = new Pixbuf (thumb_path, -1, ThumbSize);
-					thumb_cache.AddThumbnail (thumb_path, current);
+					thumb_cache.Add (thumb_path, current);
 				} catch {
 					try {
 						current = FSpot.Global.IconTheme.LoadIcon ("gtk-missing-image", ThumbSize, (Gtk.IconLookupFlags)0);
 					} catch {
 						current = null;
 					}
+					thumb_cache.Add (thumb_path, null);
 				}
-			}
 
+			}
+			
+			//FIXME
+			if (FSpot.ColorManagement.IsEnabled) {
+				current = current.Copy ();
+				FSpot.ColorManagement.ApplyScreenProfile (current);
+			}
+			
 			if (!highlighted)
 				return current;
 
@@ -531,20 +581,36 @@ namespace FSpot.Widgets
 
 		~Filmstrip ()
 		{
-			this.Dispose ();
+			Log.DebugFormat ("Finalizer called on {0}. Should be Disposed", GetType ());		
+			Dispose (false);
 		}
 			
 		public override void Dispose ()
 		{
-			lock (this) {
+			Dispose (true);
+			base.Dispose ();
+			System.GC.SuppressFinalize (this);
+		}
+
+		bool is_disposed = false;
+		protected virtual void Dispose (bool disposing)
+		{
+			if (is_disposed)
+				return;
+			if (disposing) {
+				this.selection.Changed -= HandlePointerChanged;
+				this.selection.Collection.Changed -= HandleCollectionChanged;
+				this.selection.Collection.ItemsChanged -= HandleCollectionItemsChanged;
+				ThumbnailGenerator.Default.OnPixbufLoaded -= HandlePixbufLoaded;
 				if (background_pixbuf != null)
 					background_pixbuf.Dispose ();
 				if (background_tile != null)
 					background_tile.Dispose ();
+				thumb_cache.Dispose ();
 			}
-			background_pixbuf = null;
-			background_tile = null;
-			System.GC.SuppressFinalize (this);
+			//Free unmanaged resources
+
+			is_disposed = true;
 		}
 
 		public interface IAnimator
@@ -607,7 +673,7 @@ namespace FSpot.Widgets
 		{
 			PositionChangedHandler handler;
 			Filmstrip filmstrip;
-			uint interval = 10;
+			uint interval = 50;
 			float target;
 			float speed;
 			float acc = 50f; //images/second^2
@@ -621,7 +687,7 @@ namespace FSpot.Widgets
 			public void MoveTo (float target)
 			{
 				this.target = target;
-				GLib.Timeout.Add (interval, new GLib.TimeoutHandler (Step)); 	
+				GLib.Timeout.Add (interval, new GLib.TimeoutHandler (Step));
 			}
 
 			bool Step ()
@@ -647,7 +713,7 @@ namespace FSpot.Widgets
 
 				float increment = speed * interval / 1000f;
 
-				if (Math.Abs (distance - increment) < 0.01) {
+				if (Math.Abs (distance - increment) < 0.4) {
 					handler (target);
 					return false;
 				}
