@@ -17,6 +17,7 @@ using FSpot.Xmp;
 using SemWeb;
 using SemWeb.Util;
 using Mono.Unix;
+using FSpot.Utils;
 
 namespace FSpot.Xmp {
         internal class XmpTagsImporter {
@@ -43,6 +44,8 @@ namespace FSpot.Xmp {
 		const string People = MetadataStore.IViewNS + "People";
 		const string Subject = MetadataStore.DcNS + "subject";
 		const string RdfType = MetadataStore.RdfNS + "type";
+		const string Rating = MetadataStore.XmpNS + "Rating";
+		const string Urgency = MetadataStore.PhotoshopNS + "Urgency";
 		
 		private class TagInfo {
 			// This class contains the Root tag name, and its Icon name (if any)
@@ -81,7 +84,6 @@ namespace FSpot.Xmp {
 		{
 			this.tag_store = tag_store;
 			tags_created = new Stack ();
-			// Prepare the Last Import root tag
 			
 			li_root_tag = new TagInfo (Catalog.GetString ("Import Tags"), LastImportIcon);
 			taginfo_table [(Entity)Location] = new TagInfo (Catalog.GetString ("Location"), PlacesIcon);
@@ -112,7 +114,7 @@ namespace FSpot.Xmp {
 		private static string GetTextField (Resource value)
 		{
 			string text_field = null;
-			Literal l = value as Literal;
+			SemWeb.Literal l = value as SemWeb.Literal;
 			if (l != null && l.Value != null && l.Value.Length > 0) {
 				text_field = l.Value;
 				if (Char.IsControl (l.Value [l.Value.Length - 1]))
@@ -149,17 +151,18 @@ namespace FSpot.Xmp {
 		public void ProcessStore (MetadataStore store, Photo photo)
 		{
 			Hashtable descriptions = new Hashtable ();
+			uint rating = System.UInt32.MaxValue; 
+			uint urgency = System.UInt32.MaxValue;
 
 			foreach (Statement stmt in store) {
 				//StatementList list = null;
 				
-				//System.Console.WriteLine ("processing {0}", stmt);
 				switch (stmt.Predicate.Uri) {
 
 				case Caption:
 				case Headline:
 					if (!descriptions.Contains (stmt.Predicate.Uri)) {
-						string caption = GetTextField (stmt.Object as Literal);
+						string caption = GetTextField (stmt.Object as SemWeb.Literal);
 						if (caption != null)
 							caption = caption.Trim ();
 
@@ -177,7 +180,7 @@ namespace FSpot.Xmp {
 						if ( (tag.Predicate != RdfType) && (!descriptions.Contains (stmt.Predicate.Uri)) ) {
 							string title = null;
 							try {
-								title = (GetTextField ((Literal) tag.Object)).Trim ();
+								title = (GetTextField ((SemWeb.Literal) tag.Object)).Trim ();
 							} catch {
 							}
 							if ( (title != null) && (title.Length > 0) )
@@ -186,12 +189,34 @@ namespace FSpot.Xmp {
 					}
 					break;
 
+				case Urgency: // Used if Rating was not found
+				case Rating:
+					SemWeb.Literal l = stmt.Object as SemWeb.Literal;
+					if (l != null && l.Value != null && l.Value.Length > 0) {
+						uint tmp_ui;
+						try {
+							tmp_ui = System.Convert.ToUInt32 (l.Value);
+						} catch {
+							// Set rating to 0, and continue
+							Log.DebugFormat ("Found illegal rating >{0}< in predicate {1}. Rating cleared",
+										 l.Value, stmt.Predicate.Uri);
+							tmp_ui = 0;
+						}
+						if (tmp_ui > 5) // Max rating allowed in F-Spot
+							tmp_ui = 5;
+						if (stmt.Predicate.Uri == Rating)
+							rating = tmp_ui;
+						else
+							urgency = tmp_ui == 0 ? 0 : tmp_ui - 1; // Urgency valid values 1 - 8
+					}	 
+					break;
+
 				case State:
 				case City:
 				case Country:
 				case Location:
 				case Source:
-					AddTagToPhoto (photo, stmt.Object as Literal, taginfo_table [stmt.Predicate] as TagInfo);
+					AddTagToPhoto (photo, stmt.Object as SemWeb.Literal, taginfo_table [stmt.Predicate] as TagInfo);
 					break;
 					
 				case Subject:
@@ -203,7 +228,7 @@ namespace FSpot.Xmp {
 					foreach (Statement tag in store.Select (new Statement (stmt.Object as Entity, null, null))) {
 						
 						if (tag.Predicate != RdfType)
-							AddTagToPhoto (photo, tag.Object as Literal, null);
+							AddTagToPhoto (photo, tag.Object as SemWeb.Literal, null);
 
 					}
 					break;
@@ -213,6 +238,12 @@ namespace FSpot.Xmp {
 			if (descriptions.Contains (UserComment))
 				photo.Description = descriptions [UserComment] as String;
 
+			// Use the old urgency, only if rating was not available.
+			if (urgency < System.UInt32.MaxValue)
+				photo.Rating = urgency;
+			if (rating < System.UInt32.MaxValue)
+				photo.Rating = rating;
+
 #if false	
 			//FIXME: looks like we are doing some questionable repurposing of tags here...
 
@@ -263,7 +294,10 @@ namespace FSpot.Xmp {
 			using (ImageFile img = ImageFile.Create (path)) {
 				StatementSource source = img as StatementSource;
 				if (source != null) {
-					source.Select (xmp);
+					try {
+						source.Select (xmp);
+					} catch  {
+					}
 				}
 			}
 
